//===-- MachineFunctionProvider.h - Creates MachineFunctions -----*-C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file was developed by Gordon Henriksen and is distributed under the
// University of Illinois Open Source License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// The MachineFunctionProvider "analysis" stores the current MachineFunction for
// MachineFunctionPasses. It is populated by the instruction selector and 
// destroyed by the PassManager when no longer needed.
// 
//===----------------------------------------------------------------------===//

#ifndef LLVM_CODEGEN_MACHINE_FUNCTION_PROVIDER_H
#define LLVM_CODEGEN_MACHINE_FUNCTION_PROVIDER_H

#include "llvm/Pass.h"

namespace llvm {

class MachineFunction;
class Function;
class TargetLowering;
class AnalysisUsage;

/// This class is little more than a holder for a variable. Instruction
/// selection uses makeMF to construct the MachineFunction; afterward, getMF is
/// valid until the PassManager invokes releaseMemory.
/// 
/// Since this analysis does not itself create the MachineFunction, it is vital
/// that it be preserved by all MachineFunctionPasses. MachineFunctionPass::get-
/// AnalysisUsage does this automatically, so overrides of that method should be
/// certain to call the superclass implementation.
/// 
/// Furthermore, an instruction selector must be scheduled before any
/// MachineFunctionPass.
class MachineFunctionProvider : public FunctionPass {
  Function *CurrentFn;        ///< Current function (from runOnFunction)
  MachineFunction *CurrentMF; ///< Corresponding machine function (from makeMF)
  
public:
  MachineFunctionProvider();
  ~MachineFunctionProvider();
  
  virtual void getAnalysisUsage(AnalysisUsage &AU) const;
  virtual bool runOnFunction(Function &Fn);
  virtual void releaseMemory();

  /// Used by the instruction selector to construct the machine function.
  /// Asserts if makeMF has already been called for the current Function.
  MachineFunction &makeMF(TargetMachine &TM);
  
  /// Used by MachineFunctionPass to retrieve the existing machine function.
  /// Asserts unless makeMF has been called for the current Function, or if the
  /// analysis was invalidated by another pass.
  MachineFunction &getMF() const;
};

MachineFunctionProvider *createMachineFunctionProvider();

} // End llvm namespace

#endif
